#include "Complex.h"
#include <cmath>
#include <iostream>
using namespace std;

Complex::Complex(string var, int re, int im) {
	this->re = re;
	this->im = im;
	this->var = var;
}

int Complex::fetchName(string z) {
	var = ""; int i = 0;
	// Vrtimo se kroz for petlju dok ne dodjemo do '=' jer nam to oznacava da smo stigli do kraja imena
	for (i = 0; i < z.length() && z[i] != '='; i++) {
		if (z[i] == ' ') continue; // opciono preskacemo razmake
		var += z[i];			   // svaki karakter dodajemo u ime nase promenljive jedan po jedan
	}
	// Na poziciji 'i' ce se naci znak '=', a posto nam taj znak nije od interesa, kao povratnu vrednost vracamo sledecu poziciju u stringu
	return i + 1;
}

// Metoda koja preskace razmake
int Complex::skipSpaces(string z, int k) {
	for (; k < z.length() && z[k] == ' '; k++);
	return k;
}

int Complex::fetchReal(string z, int i) {
	int k = i;
	bool minus = false;
	k = skipSpaces(z, k);
	// Opciono pre realnog dela kompleksnog broja moze da dodje znak '-' pa proveravamo tu situaciju
	if (z[k] == '-') {
		minus = true; k++;
	}
	k = skipSpaces(z, k);
	re = 0;
	// Nasli znak minus ili ne, sledece sto citamo je broj
	for (; k < z.length() && z[k] <= '9' && z[k] >= '0'; k++) {
		if (z[k] == ' ') continue;
		// U svakoj iteraciji dodajemo novu cifru na realan deo broja tako sto trenutni broj mnozimo sa 10 i dodajemo novu cifru
		re = re * 10 + (z[k] - '0'); // z[k] - '0' vraca cifru u int formatu
	}
	// Ako smo prethodno naisli na minus, menjamo znak realnog dela
	if (minus) re = -re;
	// Ukoliko nakon broja naidjemo na znak 'i', to znaci da realan deo nije ni postojao, nego smo zapravo procitali imaginaran deo
	if (z[k] == 'i' && re != 0) {
		im = re; re = 0;
	}
	else if (z[k] == 'i' && re == 0) {
		im = (minus == true) ? -1 : 1; 
		re = 0;
	}
	return k;
} 

int Complex::fetchImag(string z, int i) {
	// Metoda koja cita imaginarni deo kompleksnog broja je slicna kao i metoda koja cita realan deo broja
	int k = i;
	bool minus = false;
	k = skipSpaces(z, k);
	if (z[k] == '-') {
		minus = true; k++;
	}
	else if (z[k] == '+') {
		k++;
	}
	k = skipSpaces(z, k);
	im = 0;
	for (; k < z.length() && z[k] <= '9' && z[k] >= '0'; k++) {
		if (z[k] == ' ') continue;
		im = im * 10 + (z[k] - '0');
	}
	if (im == 0 && z[k] == 'i') im = 1;
	if (minus) im = -im;
	return k;
}

Complex::Complex(string z) {
	int i = 0; re = im = 0; var = "";
	// Vrsimo algoritamsku dekompoziciju i delimo parsiranje ovog stringa na nekoliko delova (ucitavanje promenljive, ucitavanje realnog dela i ucitavanje imaginarnog dela)
	// Svaka od ovih metoda kao parametar prima string i poziciju do koje je stigla sa citanjem stringa, a kao povratnu vrednost vracaju poziciju do koje su stigli sa citanjem stringa da bi sledeca metoda mogla da koristi tu poziciju
	i = fetchName(z);
	i = fetchReal(z, i);
	// Posto postoji mogucnost da metoda fetchReal procita imaginarni deo broja, prvo proveravamo da li ima smisla pozivati metodu fetchImag
	if(i < z.length() - 1) fetchImag(z, i);
}

int Complex::getReal() {
	return this->re;
}

int Complex::getImag() {
	return this->im;
}

double Complex::abs() {
	return sqrt(this->re * this->re + this->im * this->im);
}

double Complex::arg() {
	return im != 0 || re != 0 ? atan2(im, re) : 0;
}

void Complex::print() {
	cout << var << " = ";
	if (re == 0 && im == 0) cout << "0";
	if (re != 0) cout << re;
	if (im == 1 || im == -1) cout << (im > 0 && re != 0 ? "+" : "") << (im == 1 ? "i" : "-i");
	else if (im != 0) cout << (im > 0 && re != 0 ? "+" : "") << im << "i";
	cout << endl;
}

Complex* Complex::add(string var, Complex* z1, Complex* z2) {
	int re, im;
	re = z1->re + z2->re;
	im = z1->im + z2->im;
	return new Complex(var, re, im);
}

Complex* Complex::sub(string var, Complex* z1, Complex* z2) {
	int re, im;
	re = z1->re - z2->re;
	im = z1->im - z2->im;
	return new Complex(var, re, im);
}
